/***************************************************************************
 *   Copyright (C) 2004 by zhong                                           *
 *   zhongz@163.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
 
#include <gtk/gtk.h>
#include "interface.h"
#include "charset.h"
#include "callbacks.h"
#include "chmsee_config.h"

typedef struct
{
  const gchar    *cchar_number;
  const gchar    *cchar_codeset;
} ChmseeChar;

static ChmseeChar allchar[] =
{
	{ N_("Simplified Chinese (GB18030)"), "GB18030" },
	{ N_("Simplified Chinese (GB2312)"), "GB2312" },
	{ N_("Simplified Chinese (GBK)"), "GBK" },
	{ N_("Traditional Chinese (Big5)"), "BIG5" },
	{ N_("Western (ISO-8859-1)"), "ISO-8859-1" },
	{ N_("Western (ISO-8859-15)"), "ISO-8859-15" },
	{ N_("Western (Windows-1252)"), "Windows-1252" },
	{ N_("Japanese (Shift-JIS)"), "shift-jis" }
};


void 
on_chmsee_char_okbtn(GtkWidget *widget, gpointer data) {
	
	ChmSee				*window = (ChmSee *) data;
	GtkTreeView			*treeview = GTK_TREE_VIEW(window->chmsee_char_treeview);
	GtkTreeModel 		*model = gtk_tree_view_get_model (treeview);
	GtkTreeSelection 	*selection = gtk_tree_view_get_selection (treeview);
	GtkTreeIter 		iter;
	GValue				id_value = {0,};

	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
		
		gtk_tree_model_get_value (model, &iter, FILENAME_COLUMN, &id_value);
		window->current_codeset = g_strdup_printf("%s", g_value_get_string (&id_value));
		chmsee_go_ex_chm_file(window, window->chm_name);
		g_value_unset (&id_value);
		
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (window->chmsee_char_checkbtn))) {
        	
			/* 开关按钮是按下的 */
			window->is_always = 1;
			chmsee_save_config(window);
    
    	} else {
    
			if (window->is_always == 1) {
				window->is_always = 0;
				chmsee_save_config(window);
			}
    	}
		gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    } else {
		
		gtk_widget_destroy (gtk_widget_get_toplevel (widget));
	}

}

void 
chmsee_char_dialog_new (ChmSee *window) {

  GtkWidget *dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox2;
  GtkWidget *scrolledwindow2;
  GtkWidget *label2;
  GtkWidget *dialog_action_area1;
  GtkWidget *chmsee_char_okbtn;
  GtkTreeStore *char_model;
  GtkTreeSelection *char_selection;
  GtkCellRenderer *char_cell;
  GtkTreeViewColumn *char_column;
  gint i;
  GtkTreeIter iter;

  dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
  gtk_window_set_title (GTK_WINDOW (dialog), _("Encoding"));
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (dialog), 300, 400);

  dialog_vbox1 = GTK_DIALOG (dialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox2, TRUE, TRUE, 0);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow2);
  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow2, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_SHADOW_IN);


	char_model = gtk_tree_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	window->chmsee_char_treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (char_model));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (window->chmsee_char_treeview), TRUE);
	gtk_widget_show(window->chmsee_char_treeview);
	char_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->chmsee_char_treeview));
	gtk_tree_selection_set_mode (GTK_TREE_SELECTION (char_selection),
				     GTK_SELECTION_SINGLE);

/*	
	g_signal_connect (window->chmsee_char_treeview, "row_activated", G_CALLBACK (on_chmsee_char_okbtn), window); 
*/

	char_cell = gtk_cell_renderer_text_new ();
	char_column = gtk_tree_view_column_new_with_attributes ("CharSET",
							   char_cell,
							   "text", TITLE_COLUMN,
							   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (window->chmsee_char_treeview),
				     GTK_TREE_VIEW_COLUMN (char_column));

	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (window->chmsee_char_treeview), FALSE);


	for (i=0;i < G_N_ELEMENTS (allchar); i++) {
	
		gtk_tree_store_append(GTK_TREE_STORE (char_model), &iter, NULL);
		gtk_tree_store_set (GTK_TREE_STORE (char_model),
							&iter,
							TITLE_COLUMN, allchar[i].cchar_number,
							FILENAME_COLUMN, allchar[i].cchar_codeset,
							-1);
	}

  gtk_widget_show (window->chmsee_char_treeview);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), window->chmsee_char_treeview);

  window->chmsee_char_checkbtn = gtk_check_button_new_with_mnemonic (_("Open with Selected Code"));
  gtk_widget_show (window->chmsee_char_checkbtn);
  gtk_box_pack_start (GTK_BOX (vbox2), window->chmsee_char_checkbtn, FALSE, FALSE, 2);

  if (window->is_always == 1) {
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (window->chmsee_char_checkbtn), TRUE);
  }

  label2 = gtk_label_new (_("Select Encoding"));
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox2), label2, FALSE, FALSE, 2);

  dialog_action_area1 = GTK_DIALOG (dialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  chmsee_char_okbtn = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (chmsee_char_okbtn);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), chmsee_char_okbtn, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (chmsee_char_okbtn, GTK_CAN_DEFAULT);

  g_signal_connect_swapped ((gpointer) dialog, "destroy",
                            G_CALLBACK (gtk_widget_destroy),
                            GTK_OBJECT (dialog));
  g_signal_connect ((gpointer) chmsee_char_okbtn, "clicked",
                     G_CALLBACK (on_chmsee_char_okbtn),
                     window);
							
  gtk_widget_show(dialog);
}
