/***************************************************************************
 *   Copyright (C) 2003 by zhong                                           *
 *   zhongz@163.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "interface.h"
#include "callbacks.h"
#include "pixmaps.h"
#include "html.h"
#include "setup.h"
#include "search.h"
#include "chmsee_config.h"
#include "bookmarks.h"
#include "charset.h"

static void on_hide_pw_button (GtkWidget *widget, gpointer data);
static void on_show_pw_button (GtkWidget *widget, gpointer data);
static void on_about_open (GtkWidget *widget, gpointer data);
static GtkWidget *CreateWidgetFromXpm (GtkWidget * parent, gchar ** xpm_data);
static void nb_search_entry_changed (GtkEntry *entry, ChmSee *window);
static void nb_bookmark_entry_changed (GtkEntry *entry, ChmSee *window);

gint destroy_chmsee (GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
	return (FALSE);
}

static void
on_hide_pw_button (GtkWidget *widget, gpointer data)
{
	ChmSee *window = (ChmSee *) data;
	gtk_widget_hide( window->chm_pw );
	gtk_widget_show( window->chmsee_toolbar_showw );
}

static void
on_show_pw_button (GtkWidget *widget, gpointer data)
{
	gtk_widget_show( GTK_WIDGET(data) );
	gtk_widget_hide( widget );
}

static GtkWidget *
CreateWidgetFromXpm (GtkWidget * parent, gchar ** xpm_data)
{
	GdkBitmap *mask;
	GdkPixmap *pixmap_data;
	GtkWidget *pixmap_widget;
	GtkStyle *style;
	GdkColormap *button_color;

	button_color = gdk_colormap_get_system();
	style = gtk_widget_get_style (parent);
	pixmap_data = gdk_pixmap_colormap_create_from_xpm_d (parent->window, button_color,
						    &mask,
						    &style->bg[GTK_STATE_NORMAL],
						    (gchar **) xpm_data);
	pixmap_widget = gtk_image_new_from_pixmap (pixmap_data, NULL);
	gtk_widget_show (pixmap_widget);
	return (pixmap_widget);
}

static void 
on_about_open (GtkWidget *widget, gpointer data)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *scrolledwindow1;
  GtkWidget *textview1;
  GtkWidget *dialog_action_area1;
  GtkWidget *okbutton1;

  dialog1 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog1), "About...");
  gtk_container_set_border_width (GTK_CONTAINER (dialog1), 5);
  gtk_window_set_default_size (GTK_WINDOW (dialog1), 270, 150);
  gtk_window_unmaximize(GTK_WINDOW(dialog1));

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_show (dialog_vbox1);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_SHADOW_IN);

  textview1 = gtk_text_view_new ();
  gtk_widget_show (textview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), textview1);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview1), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview1), FALSE);
  gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview1)),
	"ChmSee (gtkhtml3)\nVersion : 0.9.5\ndeveloper : zhong \nHomepage : http://211.92.88.40/~zhong\n\n the HTML Help(CHM) viewer for Linux\n", -1);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect_swapped ((gpointer) okbutton1, "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            GTK_OBJECT (dialog1));
   gtk_widget_show(dialog1);
}

/*
static void 
on_about_open (GtkWidget *widget, ChmSee *window) {
	
	gchar	*open_uri;

	open_uri = g_strdup_printf("%s%s", PACKAGE_DATA_DIR, "/chmsee/chmsee.html");
	chm_html_open_uri(window, open_uri, 2);
}

void on_chmsee_SearchDisplay (GtkTreeView       *tree_view,
							  GtkTreePath       *path,
				 			  GtkTreeViewColumn *column,
				 			  gpointer          user_data) {

	GtkTreeModel		*model = gtk_tree_view_get_model (tree_view);
	GtkTreeIter			iter;
	gchar				*author;

	gtk_tree_model_get_iter (model, &iter, path);
	gtk_tree_model_get (model, &iter, FILENAME_COLUMN, &author, -1);
}
*/

static void 
nb_search_entry_changed (GtkEntry *entry, ChmSee *window) {
	
	const gchar *text;
	gint text_length;

	text = gtk_entry_get_text (GTK_ENTRY (entry));
	text_length = strlen(text);
	
	if (text_length >= 2) {
		gtk_widget_set_sensitive (window->nb_button_contnts, TRUE);
	} else {
		gtk_widget_set_sensitive (window->nb_button_contnts, FALSE);
	}
}

static void 
nb_bookmark_entry_changed (GtkEntry *entry, ChmSee *window) {
	
	const gchar *text;
	gint text_length;

	text = gtk_entry_get_text (GTK_ENTRY (entry));
	text_length = strlen(text);
	window->bookmark_title = (gchar *) text;

	if (text_length >= 2) {
		gtk_widget_set_sensitive (window->nb_button_bookmark_add, TRUE);

	} else {
		gtk_widget_set_sensitive (window->nb_button_bookmark_add, FALSE);
	}
}

void create_main_window (ChmSee *window)
{
  GtkWidget *vbox1;
  GtkWidget *handlebox1;
  GtkWidget *toolbar1;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *chm_open;
  GtkWidget *chm_setup;
  GtkWidget *chm_about;
  GtkWidget *chm_quit;
  GtkWidget *hpaned1;
  GtkWidget	*html_view_sw;
  GtkWidget	*chm_tree_sw;
  GtkWidget *pixmapwid, *pixgui_button, *pixgui_vbox1;
  GtkWidget *pixgui_hbox, *pixgui_frame;
  GtkWidget *hseparator1, *hseparator2, *hseparator3;
  GtkTreeStore *model;
  GtkTreeSelection *selection;
  GtkCellRenderer *cell;
  GtkTreeViewColumn *column;
  GtkTooltips *tooltips;

//notebook
  GtkWidget *notebook;
  GtkWidget *nb_label_contents;
  GtkWidget *nb_vbox1;
//  GtkWidget *nb_button_contnts;
  GtkWidget *nb_sw;
  GtkWidget *nb_treeview;
  GtkWidget *nb_button_display;
  GtkWidget *nb_label_search;
// end notebook

  GdkPixbuf    *icon;

  tooltips = gtk_tooltips_new ();

  window->chmsee_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_default_size (GTK_WINDOW (window->chmsee_window), 640, 480);
  gtk_window_maximize(GTK_WINDOW (window->chmsee_window));

  icon = gdk_pixbuf_new_from_file (PACKAGE_DATA_DIR "/pixmaps/chmsee.png", NULL);
 
	if (icon) {
		gtk_window_set_icon (GTK_WINDOW (window->chmsee_window), icon);
		g_object_unref (icon);
	}

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window->chmsee_window), vbox1);

  handlebox1 = gtk_handle_box_new ();
  gtk_widget_show (handlebox1);
  gtk_box_pack_start (GTK_BOX (vbox1), handlebox1, FALSE, FALSE, 0);

  toolbar1 = gtk_toolbar_new ();
  gtk_widget_show (toolbar1);
  gtk_container_add (GTK_CONTAINER (handlebox1), toolbar1);
  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_ICONS);

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-index", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  window->chmsee_toolbar_showw = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Show",
                                "Show Side Window", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_hide (window->chmsee_toolbar_showw);

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-open", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  chm_open = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Open"),
                                _("Open File"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (chm_open);
  
  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-find", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  window->chmsee_toolbar_search = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Find"),
                                _("Find in This Page"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (window->chmsee_toolbar_search);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-go-back", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  window->chmsee_toolbar_back = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Back"),
                                _("Back"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (window->chmsee_toolbar_back);

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-go-forward", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  window->chmsee_toolbar_forward = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Forward"),
                                _("Forward"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (window->chmsee_toolbar_forward);
  
  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-home", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  window->chmsee_toolbar_home = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Home"),
                                _("Home"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (window->chmsee_toolbar_home);

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-zoom-in", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  window->chmsee_toolbar_big = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Zoom out"),
                                _("Zoom out"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (window->chmsee_toolbar_big);

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-zoom-100", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  window->chmsee_toolbar_reset = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Normal"),
                                _("Normal"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (window->chmsee_toolbar_reset);

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-zoom-out", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  window->chmsee_toolbar_small = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Zoom in"),
                                _("Zoom in"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (window->chmsee_toolbar_small);
 
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-preferences", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  chm_setup = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Setup"),
                                _("Setup"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show(chm_setup);

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-dialog-info", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  chm_about = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("About"),
                                _("About..."), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (chm_about);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-quit", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
  chm_quit = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Quit"),
                                _("Quit"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
  gtk_widget_show (chm_quit);

  hpaned1 = gtk_hpaned_new ();
  gtk_widget_show (hpaned1);
  gtk_box_pack_start (GTK_BOX (vbox1), hpaned1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hpaned1), 2);
  gtk_paned_set_position (GTK_PANED (hpaned1), 200);

	window->chm_pw = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (window->chm_pw);
	gtk_widget_show (window->chm_pw);

	pixgui_hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (pixgui_hbox);
	gtk_box_pack_start (GTK_BOX (window->chm_pw), pixgui_hbox, FALSE, FALSE, 0);

	pixgui_frame = gtk_frame_new (NULL);
	gtk_widget_show (pixgui_frame);
	gtk_box_pack_start (GTK_BOX (pixgui_hbox), pixgui_frame, TRUE, TRUE, 1);
	gtk_container_set_border_width (GTK_CONTAINER (pixgui_frame), 0);
	gtk_frame_set_shadow_type (GTK_FRAME (pixgui_frame), GTK_SHADOW_NONE);

	pixgui_vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (pixgui_vbox1);
	gtk_container_add (GTK_CONTAINER (pixgui_frame), pixgui_vbox1);

	hseparator1 = gtk_hseparator_new ();
	gtk_widget_show (hseparator1);
	gtk_box_pack_start (GTK_BOX (pixgui_vbox1), hseparator1, TRUE, TRUE, 0);

	hseparator2 = gtk_hseparator_new ();
	gtk_widget_show (hseparator2);
	gtk_box_pack_start (GTK_BOX (pixgui_vbox1), hseparator2, TRUE, TRUE, 0);

	hseparator3 = gtk_hseparator_new ();
	gtk_widget_show (hseparator3);
	gtk_box_pack_start (GTK_BOX (pixgui_vbox1), hseparator3, TRUE, TRUE, 0);

	pixgui_button = gtk_button_new ();
	gtk_button_set_relief (GTK_BUTTON(pixgui_button),GTK_RELIEF_NONE);
	gtk_widget_show (pixgui_button);
	gtk_box_pack_start (GTK_BOX (pixgui_hbox), pixgui_button, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (pixgui_button), 0);
	gtk_tooltips_set_tip (tooltips, pixgui_button,
						  _("Hide"),
						  NULL);
    pixmapwid = CreateWidgetFromXpm(window->chmsee_window, (gchar **)hide_xpm);
    gtk_widget_show (pixmapwid);
	gtk_container_add (GTK_CONTAINER (pixgui_button), pixmapwid);

/* -----------------------------------------------------------------------------
		notebook
   -------------------------------------------------------------------------- */
  notebook = gtk_notebook_new ();
  gtk_widget_show (notebook);
  gtk_container_set_border_width (GTK_CONTAINER (notebook), 1);

/* tree */
	model = gtk_tree_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	window->chm_tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
    gtk_widget_show(window->chm_tree);
	
	g_signal_connect (window->chm_tree, "row_activated", G_CALLBACK (contents_row_cb), window);
	
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->chm_tree));
	gtk_tree_selection_set_mode (GTK_TREE_SELECTION (selection),
				     GTK_SELECTION_SINGLE);

	g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (contents_selection_cb), window);

	cell = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Tree",
							   cell,
							   "text", TITLE_COLUMN,
							   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (window->chm_tree),
				     GTK_TREE_VIEW_COLUMN (column));

	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (window->chm_tree), FALSE);
  GTK_WIDGET_SET_FLAGS (window->chm_tree, GTK_CAN_DEFAULT);

  chm_tree_sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (chm_tree_sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (chm_tree_sw), GTK_SHADOW_IN);
  gtk_container_add (GTK_CONTAINER (chm_tree_sw), window->chm_tree);
  gtk_container_set_border_width (GTK_CONTAINER (chm_tree_sw), 1);
  gtk_widget_show (chm_tree_sw);
  
  gtk_container_add (GTK_CONTAINER (notebook), chm_tree_sw);
  gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), chm_tree_sw,
                                      FALSE, TRUE, GTK_PACK_START);
// end tree

  nb_label_contents = gtk_label_new (_("Contents"));
  gtk_widget_show (nb_label_contents);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0), nb_label_contents);

  nb_vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (nb_vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), nb_vbox1);
 
  gtk_container_set_border_width (GTK_CONTAINER (nb_vbox1), 3);
  
  window->nb_entry = gtk_entry_new ();
  gtk_widget_show (window->nb_entry);
  gtk_box_pack_start (GTK_BOX (nb_vbox1), window->nb_entry, FALSE, FALSE, 2);
  gtk_widget_set_sensitive (window->nb_entry, FALSE);
  g_signal_connect (window->nb_entry, "changed", 
			  		G_CALLBACK (nb_search_entry_changed),
			  		window);

  window->nb_button_contnts = gtk_button_new_from_stock ("gtk-find");
  gtk_widget_show (window->nb_button_contnts);
  gtk_box_pack_start (GTK_BOX (nb_vbox1), window->nb_button_contnts, FALSE, FALSE, 2);
  gtk_widget_set_sensitive (window->nb_button_contnts, FALSE);

/*
  nb_vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (nb_vbox3);
  gtk_box_pack_start (GTK_BOX (nb_vbox1), nb_vbox3, FALSE, FALSE, 3);

  nb_radiobutton = gtk_radio_button_new_with_label (NULL, "在目录搜索主题");
  gtk_widget_show (nb_radiobutton);
  gtk_box_pack_start (GTK_BOX (nb_vbox3), nb_radiobutton, FALSE, FALSE, 0);

  nb_radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (nb_radiobutton));
  nb_radiobutton_search = gtk_radio_button_new_with_label (nb_radiobutton_group, "在当前页面搜索");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (nb_radiobutton_search), TRUE);
  gtk_widget_show (nb_radiobutton_search);
  gtk_box_pack_start (GTK_BOX (nb_vbox3), nb_radiobutton_search, FALSE, FALSE, 0);
*/
  nb_sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (nb_sw);
  gtk_box_pack_start (GTK_BOX (nb_vbox1), nb_sw, TRUE, TRUE, 3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (nb_sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (nb_sw), GTK_SHADOW_IN);

  nb_treeview = gtk_tree_view_new ();
  gtk_widget_show (nb_treeview);
  gtk_container_add (GTK_CONTAINER (nb_sw), nb_treeview);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (nb_treeview), FALSE);

  nb_button_display = gtk_button_new_from_stock ("gtk-jump-to");
  gtk_widget_show (nb_button_display);
  gtk_box_pack_start (GTK_BOX (nb_vbox1), nb_button_display, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (nb_button_display, FALSE);
	
  nb_label_search = gtk_label_new (_("Search"));
  gtk_widget_show (nb_label_search);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1), nb_label_search);

// bookmark

	GtkWidget *nb_vbox_bookmark;
	GtkWidget *nb_bookmark_sw;
	GtkWidget *nb_button_bookmark_remove;
	GtkWidget *nb_label_bookmark;
	GtkWidget *nb_hbox_bookmark;
	GtkTreeStore *bookmark_model;
	GtkTreeSelection *bookmark_selection;
	GtkCellRenderer *bookmark_cell;
	GtkTreeViewColumn *bookmark_column;

	nb_vbox_bookmark = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (nb_vbox_bookmark);
	gtk_container_add (GTK_CONTAINER (notebook), nb_vbox_bookmark);
	gtk_container_set_border_width (GTK_CONTAINER (nb_vbox_bookmark), 3);
  

	bookmark_model = gtk_tree_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	window->nb_bookmark_treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (bookmark_model));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (window->nb_bookmark_treeview), TRUE);
	gtk_widget_show(window->nb_bookmark_treeview);
	bookmark_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->nb_bookmark_treeview));
	gtk_tree_selection_set_mode (GTK_TREE_SELECTION (bookmark_selection),
				     GTK_SELECTION_SINGLE);

	g_signal_connect (window->nb_bookmark_treeview, "row_activated", G_CALLBACK (bookmark_selection_cb), window);

	bookmark_cell = gtk_cell_renderer_text_new ();
	bookmark_column = gtk_tree_view_column_new_with_attributes ("BookMarks",
							   bookmark_cell,
							   "text", TITLE_COLUMN,
							   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (window->nb_bookmark_treeview),
				     GTK_TREE_VIEW_COLUMN (bookmark_column));

	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (window->nb_bookmark_treeview), FALSE);
	
	nb_bookmark_sw = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (nb_bookmark_sw);
	gtk_box_pack_start (GTK_BOX (nb_vbox_bookmark), nb_bookmark_sw, TRUE, TRUE, 3);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (nb_bookmark_sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (nb_bookmark_sw), GTK_SHADOW_IN);
	gtk_container_set_border_width (GTK_CONTAINER (nb_bookmark_sw), 1);
	gtk_container_add (GTK_CONTAINER (nb_bookmark_sw), window->nb_bookmark_treeview);


  window->nb_bookmark_entry = gtk_entry_new ();
  gtk_widget_show (window->nb_bookmark_entry);
  gtk_box_pack_start (GTK_BOX (nb_vbox_bookmark), window->nb_bookmark_entry, FALSE, FALSE, 2);
//  gtk_widget_set_sensitive (window->nb_bookmark_entry, FALSE);
  g_signal_connect (window->nb_bookmark_entry, "changed", 
			  		G_CALLBACK (nb_bookmark_entry_changed),
			  		window);

  nb_hbox_bookmark = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start (GTK_BOX (nb_vbox_bookmark), nb_hbox_bookmark, FALSE, FALSE, 2);
  gtk_widget_show(nb_hbox_bookmark);

  window->nb_button_bookmark_add = gtk_button_new_from_stock ("gtk-add");
  gtk_widget_show (window->nb_button_bookmark_add);
  gtk_box_pack_start (GTK_BOX (nb_hbox_bookmark), window->nb_button_bookmark_add, TRUE, TRUE, 2);
  gtk_widget_set_sensitive (window->nb_button_bookmark_add, FALSE);

  nb_button_bookmark_remove= gtk_button_new_from_stock ("gtk-remove");
  gtk_widget_show (nb_button_bookmark_remove);
  gtk_box_pack_start (GTK_BOX (nb_hbox_bookmark), nb_button_bookmark_remove, TRUE, TRUE, 2);

  nb_label_bookmark = gtk_label_new (_("Bookmarks"));
  gtk_widget_show (nb_label_bookmark);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 2), nb_label_bookmark);

// end bookmark
// -----------------------------------------------------------------------------
	gtk_box_pack_start (GTK_BOX (window->chm_pw), notebook, TRUE, TRUE, 0);
// end notebook
	gtk_widget_hide(gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1));
// -----------------------------------------------------------------------------
	gtk_paned_pack1 (GTK_PANED (hpaned1), window->chm_pw , FALSE, TRUE);
// -----------------------------------------------------------------------------

	window->chm_html = chmsee_html_view_new(window);
	gtk_widget_show (window->chm_html);

	html_view_sw = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (html_view_sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (html_view_sw), GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER (html_view_sw), window->chm_html);
	gtk_widget_show (html_view_sw);

  gtk_paned_pack2 (GTK_PANED (hpaned1), html_view_sw, TRUE, TRUE);

  g_signal_connect ((gpointer) window->chmsee_window, "destroy",
                    G_CALLBACK (destroy_chmsee),
                    window);
  g_signal_connect ((gpointer) chm_open, "clicked",
                    G_CALLBACK (on_open_chmfile_filesel),
                    window);
  g_signal_connect ((gpointer) pixgui_button, "clicked",
                    G_CALLBACK (on_hide_pw_button),
                    window);
  g_signal_connect ((gpointer) window->chmsee_toolbar_showw, "clicked",
                    G_CALLBACK (on_show_pw_button),
                    window->chm_pw);
  g_signal_connect ((gpointer) window->chmsee_toolbar_home, "clicked",
                    G_CALLBACK (go_home),
                    window);
  g_signal_connect ((gpointer) window->chmsee_toolbar_back, "clicked",
                    G_CALLBACK (on_chm_back),
                    window);
  g_signal_connect ((gpointer) window->chmsee_toolbar_forward, "clicked",
                    G_CALLBACK (on_chm_forward),
                    window);
  g_signal_connect ((gpointer) chm_setup, "clicked",
                    G_CALLBACK (chmsee_setup_window_new),
                    window);
  g_signal_connect ((gpointer) chm_about, "clicked",
                    G_CALLBACK (on_about_open),
                    window);
  g_signal_connect ((gpointer) chm_quit, "clicked",
                    G_CALLBACK (destroy_chmsee),
                    window);
  g_signal_connect ((gpointer) window->chmsee_toolbar_big, "clicked",
                    G_CALLBACK (on_chm_big),
                    window);
  g_signal_connect ((gpointer) window->chmsee_toolbar_reset, "clicked",
                    G_CALLBACK (on_chm_reset),
                    window);
  g_signal_connect ((gpointer) window->chmsee_toolbar_small, "clicked",
                    G_CALLBACK (on_chm_small),
                    window);
  g_signal_connect ((gpointer) window->chmsee_toolbar_search, "clicked",
                    G_CALLBACK (create_search_window),
                    window);
/*
  g_signal_connect ((gpointer) nb_treeview, "row_activated",
                    G_CALLBACK (on_chmsee_SearchDisplay),
                    window);
  g_signal_connect ((gpointer) nb_button_display, "clicked",
                    G_CALLBACK (on_chmsee_SearchDisplay),
                    window);
*/				
  g_signal_connect ((gpointer) window->nb_button_bookmark_add, "clicked",
                    G_CALLBACK (chmsee_bookmark_treeview_add),
                    window);
   g_signal_connect ((gpointer) nb_button_bookmark_remove, "clicked",
                    G_CALLBACK (chmsee_bookmark_treeview_remove),
                    window);
/*
  g_signal_connect ((gpointer) search_button_forward, "clicked",
                    G_CALLBACK (chmsee_html_search_next),
                    window);
  gtk_signal_connect ((gpointer) window->chmsee_window, "selection_get",
		      G_CALLBACK (chmsee_selection_get), window->chmsee_window);
*/

//	gtk_widget_show ( window->chmsee_window );
	gtk_widget_hide ( window->chmsee_window );
//	while(gtk_events_pending())
//		gtk_main_iteration();
}
