/***************************************************************************
 *   Copyright (C) 2003 by zhong                                           *
 *   zhongz@163.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

// #include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include "interface.h"
#include "chmsee_config.h"
#include "callbacks.h"
#include "setup.h"

typedef struct _ChmSee_Setup ChmSee_Setup;
	
struct _ChmSee_Setup {
	ChmSee		*top_data;
	GtkWidget	*setup_checkbtn;
	gchar		*s;
};

static void on_setup_window_close(GtkButton *button, ChmSee_Setup *setup_data);
static void on_chmsee_setup_clear(GtkWidget *widget, gpointer data);
static gchar *get_tmpdir(void);


static void 
on_setup_window_close(GtkButton *button, ChmSee_Setup *setup_data) {
	
	ChmSee	*window = (ChmSee *) setup_data->top_data;

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (setup_data->setup_checkbtn))) {
        
		if (window->is_always == 1) {
			window->is_always = 0;
			chmsee_save_config(window);
		}
  	}
 
	gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET(button)));
}

static void
on_chmsee_setup_clear(GtkWidget *widget, gpointer data)
{
	ChmSee_Setup *setup_data = (ChmSee_Setup *) data;
	command_delete_tmpdir( setup_data->s );
}

static 
gchar *get_tmpdir(void) 
{
	gchar *set_tmpdir;
	
	set_tmpdir = g_strdup_printf("%s/chmsee_tmp-%s", g_get_tmp_dir(), g_get_user_name());
	return (set_tmpdir);
}

void
chmsee_setup_window_new (GtkWidget *widget, ChmSee *window)
{
  GtkWidget *setup_window;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *vbox2;
  GtkWidget *hbox1;
  GtkWidget *entry1;
  GtkWidget *chm_setup_clear;
  GtkWidget *alignment1;
  GtkWidget *hbox2;
  GtkWidget *image1;
  GtkWidget *label3;
  GtkWidget *label2;
  GtkWidget *label1;
  GtkWidget *hbuttonbox1;
  GtkWidget *button1;
  GtkWidget *frame2;
  GtkWidget	*vbox3;
  GtkWidget *label;

  ChmSee_Setup *setup_data;
  setup_data = g_new0(ChmSee_Setup, 1);

  setup_data->top_data = window;
  
  setup_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width (GTK_CONTAINER (setup_window), 5);
  gtk_window_set_title (GTK_WINDOW (setup_window), _("Setup"));
  gtk_window_set_position (GTK_WINDOW (setup_window), GTK_WIN_POS_CENTER);
  gtk_window_unmaximize (GTK_WINDOW(setup_window));

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (setup_window), vbox1);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, FALSE, TRUE, 0);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame1), vbox2);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

  setup_data->s = get_tmpdir();

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox1), entry1, TRUE, TRUE, 7);
  gtk_editable_set_editable (GTK_EDITABLE (entry1), FALSE);
  GTK_WIDGET_UNSET_FLAGS (entry1, GTK_CAN_FOCUS);
  gtk_entry_set_text(GTK_ENTRY(entry1), setup_data->s);

  chm_setup_clear = gtk_button_new ();
  gtk_widget_show (chm_setup_clear);
  gtk_box_pack_start (GTK_BOX (hbox1), chm_setup_clear, FALSE, FALSE, 10);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (chm_setup_clear), alignment1);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

  image1 = gtk_image_new_from_stock ("gtk-delete", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

  label3 = gtk_label_new_with_mnemonic (_("Clear"));
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 5);

  label2 = gtk_label_new (_("Please clean here regularly because some files \nwere saved while CHMSEE crash accidentally.\n\nWarning: If you are viewing CHM files, please don't clean the buffer\nbecause it may lead to CHMSEE crash."));
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox2), label2, FALSE, FALSE, 5);
  gtk_misc_set_padding (GTK_MISC (label2), 5, 5);

  label1 = gtk_label_new (_("Cache"));
  gtk_widget_show (label1);
  gtk_frame_set_label_widget (GTK_FRAME (frame1), label1);

  frame2 = gtk_frame_new (NULL);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, FALSE, TRUE, 0);

  vbox3 = gtk_vbox_new(FALSE,0);
  gtk_widget_show(vbox3);
  gtk_container_add (GTK_CONTAINER (frame2), vbox3);

  setup_data->setup_checkbtn = gtk_check_button_new_with_mnemonic (_("Show Encoding window"));
  gtk_widget_show(setup_data->setup_checkbtn);
  gtk_box_pack_start (GTK_BOX (vbox3), setup_data->setup_checkbtn, FALSE, FALSE, 2);
  
  if (window->is_always == 0) {
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (setup_data->setup_checkbtn), TRUE);
	  gtk_widget_set_sensitive (setup_data->setup_checkbtn, FALSE);
  }

  label = gtk_label_new (_("Other"));
  gtk_widget_show (label);
  gtk_frame_set_label_widget (GTK_FRAME (frame2), label);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, FALSE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);

  button1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (button1);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
  gtk_container_set_border_width (GTK_CONTAINER (button1), 6);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  g_signal_connect_swapped ((gpointer) setup_window, "destroy",
                            G_CALLBACK (gtk_widget_destroy),
                            GTK_OBJECT (setup_window));
  g_signal_connect ((gpointer) chm_setup_clear, "clicked",
                     G_CALLBACK (on_chmsee_setup_clear),
                     setup_data);
  g_signal_connect ( button1, "clicked",
                     G_CALLBACK (on_setup_window_close),
                     setup_data);

  gtk_widget_grab_focus (button1);
  gtk_widget_show(setup_window);
  gtk_window_set_transient_for (GTK_WINDOW (setup_window), GTK_WINDOW (window->chmsee_window));
}
